# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 15:29:46 2013

@author: delande
"""
# Copyright Dominique Delande
# Provided "as is" without any warranty
#
# Computes the distribution of final momenta
# for a set of trajectories of the quasi-periodically kicked rotor
# Initial conditions are generated randomly in the [-pi,pi]x[-pi,pi] square for direction 1
# and are fixed for direction 2 and 3
# The initial p2 and p3 are irrelevant and chosen equal to 0
# Prints the histogram of the distribution of final momentum
# The histogram is a Gaussian even for moderate values of K,
# as low as K=3, provided epsilon is not very small.
# This is in contrast with the standard periodically kicked rotor
# where classical structures in phase persist up to K=8
#
# 10000 trajectories produce a decent histogram
# Execution time scales like number_of_tajectories*final_time
# For 10000 trajectories up to final_time 100, it takes 10 seconds
# on a slow moderately old (2009) laptop

from math import *
import time
import socket
import os
import random
import numpy

dpi=2.0*pi
# The following set of parameters for the figure in the lecture notes (with less averaging)
# (perfect Gaussian). Warning: takes 90 seconds execution time
K=10.0
epsilon=0.8
final_time=1000
bin_size=15.0
# The following set of parameters shows the absence of accelarator modes
# (perfect Gaussian). Compare with epsilon=0
#K=7.0
#epsilon=0.3
#final_time=100
#bin_size=6.0
# The following set of parameters shows the classical diffusive process down to K=4. 
# Compare with epsilon=0
K=4.0
epsilon=0.8
final_time=100
bin_size=2.0

number_of_trajectories=10000
number_of_bins=100
# The following 4 parameters are essentially irrelevant
# as all non-pathological values produce similar histograms 
omega_2=2.67220902067
omega_3=2.01719090925
phi_2=0.4
phi_3=0.34
I_max=number_of_bins*bin_size/2.0
I_min=-I_max
population_bins_1=numpy.zeros(number_of_bins)
population_bins_2=numpy.zeros(number_of_bins)
population_bins_3=numpy.zeros(number_of_bins)
for i_traj in range(number_of_trajectories):
  x_1=random.uniform(0,dpi)
  p_1=random.uniform(0,dpi)-pi
  x_2=phi_2
  p_2=0.0
  x_3=phi_3
  p_3=0.0
  for i in range(0,final_time):
    p_1+=K*sin(x_1)*(1.0+epsilon*cos(x_2)*cos(x_3))
    p_2+=K*epsilon*cos(x_1)*sin(x_2)*cos(x_3)
    p_3+=K*epsilon*cos(x_1)*cos(x_2)*sin(x_3)
    x_1+=p_1
    x_2+=omega_2
    x_3+=omega_3
  i_bin=int(floor((p_1-I_min)/bin_size))
  if ((i_bin>=0)&(i_bin<number_of_bins)):
    population_bins_1[i_bin]+=1.0   
  i_bin=int(floor((p_2-I_min)/bin_size))
  if ((i_bin>=0)&(i_bin<number_of_bins)):
    population_bins_2[i_bin]+=1.0   
  i_bin=int(floor((p_3-I_min)/bin_size))
  if ((i_bin>=0)&(i_bin<number_of_bins)):
    population_bins_3[i_bin]+=1.0   
population_bins_1/=(bin_size*number_of_trajectories)   
population_bins_2/=(bin_size*number_of_trajectories)   
population_bins_3/=(bin_size*number_of_trajectories)   
p2_file=open('qpkr_classical_momentum_histograms.dat','w')
print >>p2_file,'# Data generated on',time.asctime(),' using Python script',os.path.abspath( __file__ ),' on computer:',socket.gethostname()
print >>p2_file,'# Final momentum distribution for the quasi-periodically kicked rotor'
print >>p2_file,'# The 3 distributions one after the other'
print >>p2_file,'# K =',K
print >>p2_file,'# epsilon =',epsilon
print >>p2_file,'# omega_2 =',omega_2
print >>p2_file,'# phi_2   =',phi_2
print >>p2_file,'# omega_3 =',omega_3
print >>p2_file,'# phi_3   =',phi_3
print >>p2_file,'# number_of_trajectories =',number_of_trajectories
print >>p2_file,'# final_time =',final_time
print >>p2_file,'# number_of_bins =',number_of_bins
print >>p2_file,'# bin_size =',bin_size
for i_bin in range(number_of_bins):
  print >>p2_file,I_min+i_bin*bin_size,population_bins_1[i_bin]
  print >>p2_file,I_min+(i_bin+1)*bin_size,population_bins_1[i_bin]
print >>p2_file,' '
for i_bin in range(number_of_bins):
  print >>p2_file,I_min+i_bin*bin_size,population_bins_2[i_bin]
  print >>p2_file,I_min+(i_bin+1)*bin_size,population_bins_2[i_bin]
print >>p2_file,' '
for i_bin in range(number_of_bins):
  print >>p2_file,I_min+i_bin*bin_size,population_bins_3[i_bin]
  print >>p2_file,I_min+(i_bin+1)*bin_size,population_bins_3[i_bin]
p2_file.close()