# -*- coding: utf-8 -*-
"""
Created on Mon Jul  1 15:29:46 2013

@author: delande
"""
# Copyright Dominique Delande
# Provided "as is" without any warranty
#
# Computes the average value of <p**2(t)> as a function of time
# for the CLASSICAL quasi-periodically kicked rotor.
# Initial conditions are generated randomly
# For a single initial condition, it displays large fluctuations
# When averaged over several initial conditions, it displays
# an average linear increase in time, characteristic of a 
# (chaotic) diffusion process, for K sufficiently large,
# say K>4.
# 
# 10 initial conditions are enough for characterizing the diffusion process
# 10000 if you want a perfect linea growth
# Execution time scales like number_of_tajectories*final_time
# For 10000 trajectories up to final_time 100, it takes 8 seconds
# on a slow moderately old (2009) laptop

from math import *
import time
import socket
import os
import random
import numpy

dpi=2.0*pi
# You may decrease K down to 4, even 3 or 2 and still
# observe diffusive behavior. The quasi-periodically kicked
# rotor has much less regularities than the periodic one
# at small K (unless epsilon=0 of course).
K=4.0
epsilon=0.8
# The omega's and phi's are essentiallt irrelevant
# You may use anything
omega_2=2.67220902067
omega_3=2.01719090925
phi_2=0.4
phi_3=0.34
number_of_trajectories=10000
final_time=100
tab_p2=numpy.zeros(final_time+1)
for i_traj in range(number_of_trajectories):
  theta=random.uniform(0,dpi)
  I=random.uniform(0,dpi)-pi
  tab_p2[0]+=I*I
  for i in range(1,final_time+1):
    local_K=K*(1.0+epsilon*cos(omega_2*i+phi_2)*cos(omega_3*i+phi_3))
    I+=local_K*sin(theta)
    theta+=I
    tab_p2[i]+=I*I   
p2_file=open('qpkr_p2_vs_time_classical.dat','w')
print >>p2_file,'# Data generated on',time.asctime(),' using Python script',os.path.abspath( __file__ ),' on computer:',socket.gethostname()
print >>p2_file,'# <p**2> versus time for the quasi-periodically kicked rotor'
print >>p2_file,'# K =',K
print >>p2_file,'# epsilon =',epsilon
print >>p2_file,'# omega_2 =',omega_2
print >>p2_file,'# phi_2   =',phi_2
print >>p2_file,'# omega_3 =',omega_3
print >>p2_file,'# phi_3   =',phi_3
print >>p2_file,'# number_of_trajectories =',number_of_trajectories
print >>p2_file,'# final_time =',final_time
for i in range(final_time+1):
  print >>p2_file,i,tab_p2[i]/number_of_trajectories
p2_file.close()